#pragma once

#include <cinttypes>

#include "driver/ledc.h"

#include "PWMChannel.hpp"




namespace CyanLight {

  const uint8_t MAX_CHANNELS = 6;
  
  const uint8_t channelGPIOS[MAX_CHANNELS] = { 27, 16, 17, 18, 19, 21 };

  class Controller {
    public:
      Controller(uint16_t baseChannel, uint8_t channelsConfigured = 1);

      void setPWM(uint8_t channel, float value);


      uint8_t getChannelCount();

      void setBaseChannel(uint16_t baseChannel);
      uint16_t getBaseChannel();

      bool setFrequency(uint32_t frq_hz);
      uint32_t getFrequency();

      bool setResolution(uint8_t res_bits);
      uint8_t getResolution();

      bool setFrqRes(uint32_t frq_hz, uint8_t res_bits);

      void publishFrqRes();
      void publishFrequency();
      void publishResolution();
      void publishBaseChannel();
      void publishChannelCount();
    private:
      uint32_t frequency = 1000;  // Hz
      uint8_t resolution =  10;  // bits

      uint16_t baseChannel;
      uint8_t channelsConfigured;
      PWMChannel **channels;

      ledc_timer_config_t timer_cfg;
  };

  float bytes2float(uint8_t *bytes);
}
